@@ -5,8 +5,18 @@
 # found in the LICENSE file.
 
 """psutil is a cross-platform library for retrieving information on
-running processes and system utilization (CPU, memory, disks, network)
-in Python.
+running processes and system utilization (CPU, memory, disks, network,
+sensors) in Python. Supported platforms:
+
+ - Linux
+ - Windows
+ - OSX
+ - Sun Solaris
+ - FreeBSD
+ - OpenBSD
+ - NetBSD
+
+Works with Python versions from 2.6 to 3.X.
 """
 
 from __future__ import division
@@ -30,6 +40,7 @@ from . import _common
 from ._common import deprecated_method
 from ._common import memoize
 from ._common import memoize_when_activated
+from ._common import wrap_numbers as _wrap_numbers
 from ._compat import callable
 from ._compat import long
 from ._compat import PY3 as _PY3
@@ -187,12 +198,12 @@ __all__ = [
     "net_io_counters", "net_connections", "net_if_addrs",           # network
     "net_if_stats",
     "disk_io_counters", "disk_partitions", "disk_usage",            # disk
-    # "sensors_temperatures", "sensors_battery",                    # sensors
+    # "sensors_temperatures", "sensors_battery", "sensors_fans"     # sensors
     "users", "boot_time",                                           # others
 ]
 __all__.extend(_psplatform.__extra__all__)
 __author__ = "Giampaolo Rodola'"
-__version__ = "5.1.3"
+__version__ = "5.3.0"
 version_info = tuple([int(num) for num in __version__.split('.')])
 AF_LINK = _psplatform.AF_LINK
 POWER_TIME_UNLIMITED = _common.POWER_TIME_UNLIMITED
@@ -272,7 +283,7 @@ class ZombieProcess(NoSuchProcess):
     """
 
     def __init__(self, pid, name=None, ppid=None, msg=None):
-        Error.__init__(self, msg)
+        NoSuchProcess.__init__(self, msg)
         self.pid = pid
         self.ppid = ppid
         self.name = name
@@ -371,10 +382,10 @@ class Process(object):
      - kill()
 
     To prevent this problem for all other methods you can:
-      - use is_running() before querying the process
-      - if you're continuously iterating over a set of Process
-        instances use process_iter() which pre-emptively checks
-        process identity for every yielded instance
+     - use is_running() before querying the process
+     - if you're continuously iterating over a set of Process
+       instances use process_iter() which pre-emptively checks
+     process identity for every yielded instance
     """
 
     def __init__(self, pid=None):
@@ -532,11 +543,11 @@ class Process(object):
     def as_dict(self, attrs=None, ad_value=None):
         """Utility method returning process information as a
         hashable dictionary.
-        If 'attrs' is specified it must be a list of strings
+        If *attrs* is specified it must be a list of strings
         reflecting available Process class' attribute names
         (e.g. ['cpu_times', 'name']) else all public (read
         only) attributes are assumed.
-        'ad_value' is the value which gets assigned in case
+        *ad_value* is the value which gets assigned in case
         AccessDenied or ZombieProcess exception is raised when
         retrieving that particular process information.
         """
@@ -627,7 +638,7 @@ class Process(object):
         # Process.parent()?
         if POSIX:
             return self._proc.ppid()
-        else:
+        else:  # pragma: no cover
             self._ppid = self._ppid or self._proc.ppid()
             return self._ppid
 
@@ -792,11 +803,11 @@ class Process(object):
         def ionice(self, ioclass=None, value=None):
             """Get or set process I/O niceness (priority).
 
-            On Linux 'ioclass' is one of the IOPRIO_CLASS_* constants.
-            'value' is a number which goes from 0 to 7. The higher the
+            On Linux *ioclass* is one of the IOPRIO_CLASS_* constants.
+            *value* is a number which goes from 0 to 7. The higher the
             value, the lower the I/O priority of the process.
 
-            On Windows only 'ioclass' is used and it can be set to 2
+            On Windows only *ioclass* is used and it can be set to 2
             (normal), 1 (low) or 0 (very low).
 
             Available on Linux and Windows > Vista only.
@@ -815,8 +826,8 @@ class Process(object):
             """Get or set process resource limits as a (soft, hard)
             tuple.
 
-            'resource' is one of the RLIMIT_* constants.
-            'limits' is supposed to be a (soft, hard)  tuple.
+            *resource* is one of the RLIMIT_* constants.
+            *limits* is supposed to be a (soft, hard)  tuple.
 
             See "man prlimit" for further info.
             Available on Linux only.
@@ -831,7 +842,7 @@ class Process(object):
 
         def cpu_affinity(self, cpus=None):
             """Get or set process CPU affinity.
-            If specified 'cpus' must be a list of CPUs for which you
+            If specified, *cpus* must be a list of CPUs for which you
             want to set the affinity (e.g. [0, 1]).
             If an empty list is passed, all egible CPUs are assumed
             (and set).
@@ -901,7 +912,7 @@ class Process(object):
     def children(self, recursive=False):
         """Return the children of this process as a list of Process
         instances, pre-emptively checking whether PID has been reused.
-        If recursive is True return all the parent descendants.
+        If *recursive* is True return all the parent descendants.
 
         Example (A == this process):
 
@@ -997,12 +1008,12 @@ class Process(object):
         """Return a float representing the current process CPU
         utilization as a percentage.
 
-        When interval is 0.0 or None (default) compares process times
+        When *interval* is 0.0 or None (default) compares process times
         to system CPU times elapsed since last call, returning
         immediately (non-blocking). That means that the first time
         this is called it will return a meaningful 0.0 value.
 
-        When interval is > 0.0 compares process times to system CPU
+        When *interval* is > 0.0 compares process times to system CPU
         times elapsed before and after the interval (blocking).
 
         In this case is recommended for accuracy that this function
@@ -1011,7 +1022,7 @@ class Process(object):
         A value > 100.0 can be returned in case of processes running
         multiple threads on different CPU cores.
 
-        The returned value is explicitly *not* split evenly between
+        The returned value is explicitly NOT split evenly between
         all available logical CPUs. This means that a busy loop process
         running on a system with 2 logical CPUs will be reported as
         having 100% CPU utilization instead of 50%.
@@ -1067,7 +1078,7 @@ class Process(object):
             # interval was too low
             return 0.0
         else:
-            # Note 1.
+            # Note 1:
             # in order to emulate "top" we multiply the value for the num
             # of CPU cores. This way the busy process will be reported as
             # having 100% (or more) usage.
@@ -1076,7 +1087,7 @@ class Process(object):
             # taskmgr.exe on Windows differs in that it will show 50%
             # instead.
             #
-            # Note #3:
+            # Note 3:
             # a percentage > 100 is legitimate as it can result from a
             # process with multiple threads running on different CPU
             # cores (top does the same), see:
@@ -1130,7 +1141,7 @@ class Process(object):
     def memory_percent(self, memtype="rss"):
         """Compare process memory to total physical system memory and
         calculate process memory utilization as a percentage.
-        'memtype' argument is a string that dictates what type of
+        *memtype* argument is a string that dictates what type of
         process memory you want to compare against (defaults to "rss").
         The list of available strings can be obtained like this:
 
@@ -1165,10 +1176,10 @@ class Process(object):
             """Return process' mapped memory regions as a list of namedtuples
             whose fields are variable depending on the platform.
 
-            If 'grouped' is True the mapped regions with the same 'path'
+            If *grouped* is True the mapped regions with the same 'path'
             are grouped together and the different memory fields are summed.
 
-            If 'grouped' is False every mapped region is shown as a single
+            If *grouped* is False every mapped region is shown as a single
             entity and the namedtuple will also include the mapped region's
             address space ('addr') and permission set ('perms').
             """
@@ -1196,23 +1207,26 @@ class Process(object):
         return self._proc.open_files()
 
     def connections(self, kind='inet'):
-        """Return connections opened by process as a list of
+        """Return socket connections opened by process as a list of
         (fd, family, type, laddr, raddr, status) namedtuples.
-        The 'kind' parameter filters for connections that match the
+        The *kind* parameter filters for connections that match the
         following criteria:
 
-        Kind Value      Connections using
-        inet            IPv4 and IPv6
-        inet4           IPv4
-        inet6           IPv6
-        tcp             TCP
-        tcp4            TCP over IPv4
-        tcp6            TCP over IPv6
-        udp             UDP
-        udp4            UDP over IPv4
-        udp6            UDP over IPv6
-        unix            UNIX socket (both UDP and TCP protocols)
-        all             the sum of all the possible families and protocols
+        +------------+----------------------------------------------------+
+        | Kind Value | Connections using                                  |
+        +------------+----------------------------------------------------+
+        | inet       | IPv4 and IPv6                                      |
+        | inet4      | IPv4                                               |
+        | inet6      | IPv6                                               |
+        | tcp        | TCP                                                |
+        | tcp4       | TCP over IPv4                                      |
+        | tcp6       | TCP over IPv6                                      |
+        | udp        | UDP                                                |
+        | udp4       | UDP over IPv4                                      |
+        | udp6       | UDP over IPv6                                      |
+        | unix       | UNIX socket (both UDP and TCP protocols)           |
+        | all        | the sum of all the possible families and protocols |
+        +------------+----------------------------------------------------+
         """
         return self._proc.connections(kind)
 
@@ -1244,8 +1258,8 @@ class Process(object):
 
     @_assert_pid_not_reused
     def send_signal(self, sig):
-        """Send a signal to process pre-emptively checking whether
-        PID has been reused (see signal module constants) .
+        """Send a signal *sig* to process pre-emptively checking
+        whether PID has been reused (see signal module constants) .
         On Windows only SIGTERM is valid and is treated as an alias
         for kill().
         """
@@ -1313,8 +1327,8 @@ class Process(object):
         If the process is already terminated immediately return None
         instead of raising NoSuchProcess.
 
-        If timeout (in seconds) is specified and process is still alive
-        raise TimeoutExpired.
+        If *timeout* (in seconds) is specified and process is still
+        alive raise TimeoutExpired.
 
         To wait for multiple Process(es) use psutil.wait_procs().
         """
@@ -1451,7 +1465,7 @@ def pid_exists(pid):
 _pmap = {}
 
 
-def process_iter():
+def process_iter(attrs=None, ad_value=None):
     """Return a generator yielding a Process instance for all
     running processes.
 
@@ -1464,9 +1478,18 @@ def process_iter():
 
     The sorting order in which processes are yielded is based on
     their PIDs.
+
+    *attrs* and *ad_value* have the same meaning as in
+    Process.as_dict(). If *attrs* is specified as_dict() is called
+    and the resulting dict is stored as a 'info' attribute attached
+    to returned Process instance.
+    If *attrs* is an empty list it will retrieve all process info
+    (slow).
     """
     def add(pid):
         proc = Process(pid)
+        if attrs is not None:
+            proc.info = proc.as_dict(attrs=attrs, ad_value=ad_value)
         _pmap[proc.pid] = proc
         return proc
 
@@ -1489,6 +1512,9 @@ def process_iter():
                 # use is_running() to check whether PID has been reused by
                 # another process in which case yield a new Process instance
                 if proc.is_running():
+                    if attrs is not None:
+                        proc.info = proc.as_dict(
+                            attrs=attrs, ad_value=ad_value)
                     yield proc
                 else:
                     yield add(pid)
@@ -1516,14 +1542,16 @@ def wait_procs(procs, timeout=None, callback=None):
     Return a (gone, alive) tuple indicating which processes
     are gone and which ones are still alive.
 
-    The gone ones will have a new 'returncode' attribute indicating
+    The gone ones will have a new *returncode* attribute indicating
     process exit status (may be None).
 
-    'callback' is a function which gets called every time a process
+    *callback* is a function which gets called every time a process
     terminates (a Process instance is passed as callback argument).
 
     Function will return as soon as all processes terminate or when
-    timeout occurs.
+    *timeout* occurs.
+    Differently from Process.wait() it will not raise TimeoutExpired if
+    *timeout* occurs.
 
     Typical use case is:
 
@@ -1600,12 +1628,11 @@ def wait_procs(procs, timeout=None, callback=None):
 # =====================================================================
 
 
-@memoize
 def cpu_count(logical=True):
     """Return the number of logical CPUs in the system (same as
     os.cpu_count() in Python 3.4).
 
-    If logical is False return the number of physical cores only
+    If *logical* is False return the number of physical cores only
     (e.g. hyper thread CPUs are excluded).
 
     Return None if undetermined.
@@ -1616,15 +1643,20 @@ def cpu_count(logical=True):
     >>> psutil.cpu_count.cache_clear()
     """
     if logical:
-        return _psplatform.cpu_count_logical()
+        ret = _psplatform.cpu_count_logical()
     else:
-        return _psplatform.cpu_count_physical()
+        ret = _psplatform.cpu_count_physical()
+    if ret is not None and ret < 1:
+        ret = None
+    return ret
 
 
 def cpu_times(percpu=False):
     """Return system-wide CPU times as a namedtuple.
-    Every CPU time represents the seconds the CPU has spent in the given mode.
-    The namedtuple's fields availability varies depending on the platform:
+    Every CPU time represents the seconds the CPU has spent in the
+    given mode. The namedtuple's fields availability varies depending on the
+    platform:
+
      - user
      - system
      - idle
@@ -1636,7 +1668,7 @@ def cpu_times(percpu=False):
      - guest (Linux >= 2.6.24)
      - guest_nice (Linux >= 3.2.0)
 
-    When percpu is True return a list of namedtuples for each CPU.
+    When *percpu* is True return a list of namedtuples for each CPU.
     First element of the list refers to first CPU, second element
     to second CPU and so on.
     The order of the list is consistent across calls.
@@ -1701,17 +1733,17 @@ def cpu_percent(interval=None, percpu=False):
     """Return a float representing the current system-wide CPU
     utilization as a percentage.
 
-    When interval is > 0.0 compares system CPU times elapsed before
+    When *interval* is > 0.0 compares system CPU times elapsed before
     and after the interval (blocking).
 
-    When interval is 0.0 or None compares system CPU times elapsed
+    When *interval* is 0.0 or None compares system CPU times elapsed
     since last call or module import, returning immediately (non
     blocking). That means the first time this is called it will
     return a meaningless 0.0 value which you should ignore.
     In this case is recommended for accuracy that this function be
     called with at least 0.1 seconds between calls.
 
-    When percpu is True returns a list of floats representing the
+    When *percpu* is True returns a list of floats representing the
     utilization as a percentage for each CPU.
     First element of the list refers to first CPU, second element
     to second CPU and so on.
@@ -1808,7 +1840,7 @@ def cpu_times_percent(interval=None, percpu=False):
                  irq=0.0, softirq=0.0, steal=0.0, guest=0.0, guest_nice=0.0)
       >>>
 
-    interval and percpu arguments have the same meaning as in
+    *interval* and *percpu* arguments have the same meaning as in
     cpu_percent().
     """
     global _last_cpu_times_2
@@ -1888,7 +1920,7 @@ if hasattr(_psplatform, "cpu_freq"):
         """Return CPU frequency as a nameduple including current,
         min and max frequency expressed in Mhz.
 
-        If percpu is True and the system supports per-cpu frequency
+        If *percpu* is True and the system supports per-cpu frequency
         retrieval (Linux only) a list of frequencies is returned for
         each CPU. If not a list with one element is returned.
         """
@@ -1898,7 +1930,7 @@ if hasattr(_psplatform, "cpu_freq"):
         else:
             num_cpus = float(len(ret))
             if num_cpus == 0:
-                return []
+                return None
             elif num_cpus == 1:
                 return ret[0]
             else:
@@ -1907,18 +1939,9 @@ if hasattr(_psplatform, "cpu_freq"):
                     currs += cpu.current
                     mins += cpu.min
                     maxs += cpu.max
-                try:
-                    current = currs / num_cpus
-                except ZeroDivisionError:
-                    current = 0.0
-                try:
-                    min_ = mins / num_cpus
-                except ZeroDivisionError:
-                    min_ = 0.0
-                try:
-                    max_ = maxs / num_cpus
-                except ZeroDivisionError:
-                    max_ = 0.0
+                current = currs / num_cpus
+                min_ = mins / num_cpus
+                max_ = maxs / num_cpus
                 return _common.scpufreq(current, min_, max_)
 
     __all__.append("cpu_freq")
@@ -1947,8 +1970,8 @@ def virtual_memory():
        the percentage usage calculated as (total - available) / total * 100
 
      - used:
-       memory used, calculated differently depending on the platform and
-       designed for informational purposes only:
+        memory used, calculated differently depending on the platform and
+        designed for informational purposes only:
         OSX: active + inactive + wired
         BSD: active + wired + cached
         LINUX: total - free
@@ -2010,9 +2033,9 @@ def swap_memory():
 
 
 def disk_usage(path):
-    """Return disk usage statistics about the given path as a namedtuple
-    including total, used and free space expressed in bytes plus the
-    percentage usage.
+    """Return disk usage statistics about the given *path* as a
+    namedtuple including total, used and free space expressed in bytes
+    plus the percentage usage.
     """
     return _psplatform.disk_usage(path)
 
@@ -2023,13 +2046,13 @@ def disk_partitions(all=False):
     'opts' field is a raw string separated by commas indicating mount
     options which may vary depending on the platform.
 
-    If "all" parameter is False return physical devices only and ignore
+    If *all* parameter is False return physical devices only and ignore
     all others.
     """
     return _psplatform.disk_partitions(all)
 
 
-def disk_io_counters(perdisk=False):
+def disk_io_counters(perdisk=False, nowrap=True):
     """Return system disk I/O statistics as a namedtuple including
     the following fields:
 
@@ -2037,20 +2060,35 @@ def disk_io_counters(perdisk=False):
      - write_count: number of writes
      - read_bytes:  number of bytes read
      - write_bytes: number of bytes written
-     - read_time:   time spent reading from disk (in milliseconds)
-     - write_time:  time spent writing to disk (in milliseconds)
+     - read_time:   time spent reading from disk (in ms)
+     - write_time:  time spent writing to disk (in ms)
 
-    If perdisk is True return the same information for every
+    Platform specific:
+
+     - busy_time: (Linux, FreeBSD) time spent doing actual I/Os (in ms)
+     - read_merged_count (Linux): number of merged reads
+     - write_merged_count (Linux): number of merged writes
+
+    If *perdisk* is True return the same information for every
     physical disk installed on the system as a dictionary
     with partition names as the keys and the namedtuple
     described above as the values.
 
+    If *nowrap* is True it detects and adjust the numbers which overflow
+    and wrap (restart from 0) and add "old value" to "new value" so that
+    the returned numbers will always be increasing or remain the same,
+    but never decrease.
+    "disk_io_counters.cache_clear()" can be used to invalidate the
+    cache.
+
     On recent Windows versions 'diskperf -y' command may need to be
     executed first otherwise this function won't find any disk.
     """
     rawdict = _psplatform.disk_io_counters()
     if not rawdict:
-        raise RuntimeError("couldn't find any physical disk")
+        return {} if perdisk else None
+    if nowrap:
+        rawdict = _wrap_numbers(rawdict, 'psutil.disk_io_counters')
     nt = getattr(_psplatform, "sdiskio", _common.sdiskio)
     if perdisk:
         for disk, fields in rawdict.items():
@@ -2060,12 +2098,17 @@ def disk_io_counters(perdisk=False):
         return nt(*[sum(x) for x in zip(*rawdict.values())])
 
 
+disk_io_counters.cache_clear = functools.partial(
+    _wrap_numbers.cache_clear, 'psutil.disk_io_counters')
+disk_io_counters.cache_clear.__doc__ = "Clears nowrap argument cache"
+
+
 # =====================================================================
 # --- network related functions
 # =====================================================================
 
 
-def net_io_counters(pernic=False):
+def net_io_counters(pernic=False, nowrap=True):
     """Return network I/O statistics as a namedtuple including
     the following fields:
 
@@ -2079,14 +2122,23 @@ def net_io_counters(pernic=False):
      - dropout:      total number of outgoing packets which were dropped
                      (always 0 on OSX and BSD)
 
-    If pernic is True return the same information for every
+    If *pernic* is True return the same information for every
     network interface installed on the system as a dictionary
     with network interface names as the keys and the namedtuple
     described above as the values.
+
+    If *nowrap* is True it detects and adjust the numbers which overflow
+    and wrap (restart from 0) and add "old value" to "new value" so that
+    the returned numbers will always be increasing or remain the same,
+    but never decrease.
+    "disk_io_counters.cache_clear()" can be used to invalidate the
+    cache.
     """
     rawdict = _psplatform.net_io_counters()
     if not rawdict:
-        raise RuntimeError("couldn't find any network interface")
+        return {} if pernic else None
+    if nowrap:
+        rawdict = _wrap_numbers(rawdict, 'psutil.net_io_counters')
     if pernic:
         for nic, fields in rawdict.items():
             rawdict[nic] = _common.snetio(*fields)
@@ -2095,26 +2147,34 @@ def net_io_counters(pernic=False):
         return _common.snetio(*[sum(x) for x in zip(*rawdict.values())])
 
 
+net_io_counters.cache_clear = functools.partial(
+    _wrap_numbers.cache_clear, 'psutil.net_io_counters')
+net_io_counters.cache_clear.__doc__ = "Clears nowrap argument cache"
+
+
 def net_connections(kind='inet'):
-    """Return system-wide connections as a list of
+    """Return system-wide socket connections as a list of
     (fd, family, type, laddr, raddr, status, pid) namedtuples.
     In case of limited privileges 'fd' and 'pid' may be set to -1
     and None respectively.
-    The 'kind' parameter filters for connections that fit the
+    The *kind* parameter filters for connections that fit the
     following criteria:
 
-    Kind Value      Connections using
-    inet            IPv4 and IPv6
-    inet4           IPv4
-    inet6           IPv6
-    tcp             TCP
-    tcp4            TCP over IPv4
-    tcp6            TCP over IPv6
-    udp             UDP
-    udp4            UDP over IPv4
-    udp6            UDP over IPv6
-    unix            UNIX socket (both UDP and TCP protocols)
-    all             the sum of all the possible families and protocols
+    +------------+----------------------------------------------------+
+    | Kind Value | Connections using                                  |
+    +------------+----------------------------------------------------+
+    | inet       | IPv4 and IPv6                                      |
+    | inet4      | IPv4                                               |
+    | inet6      | IPv6                                               |
+    | tcp        | TCP                                                |
+    | tcp4       | TCP over IPv4                                      |
+    | tcp6       | TCP over IPv6                                      |
+    | udp        | UDP                                                |
+    | udp4       | UDP over IPv4                                      |
+    | udp6       | UDP over IPv6                                      |
+    | unix       | UNIX socket (both UDP and TCP protocols)           |
+    | all        | the sum of all the possible families and protocols |
+    +------------+----------------------------------------------------+
 
     On OSX this function requires root privileges.
     """
@@ -2127,19 +2187,14 @@ def net_if_addrs():
     NIC names and value is a list of namedtuples for each address
     assigned to the NIC. Each namedtuple includes 5 fields:
 
-     - family
-     - address
-     - netmask
-     - broadcast
-     - ptp
-
-    'family' can be either socket.AF_INET, socket.AF_INET6 or
-    psutil.AF_LINK, which refers to a MAC address.
-    'address' is the primary address and it is always set.
-    'netmask' and 'broadcast' and 'ptp' may be None.
-    'ptp' stands for "point to point" and references the destination
-    address on a point to point interface (typically a VPN).
-    'broadcast' and 'ptp' are mutually exclusive.
+     - family: can be either socket.AF_INET, socket.AF_INET6 or
+               psutil.AF_LINK, which refers to a MAC address.
+     - address: is the primary address and it is always set.
+     - netmask: and 'broadcast' and 'ptp' may be None.
+     - ptp: stands for "point to point" and references the
+            destination address on a point to point interface
+            (typically a VPN).
+     - broadcast: and *ptp* are mutually exclusive.
 
     Note: you can have more than one address of the same family
     associated with each interface.
@@ -2205,8 +2260,9 @@ if hasattr(_psplatform, "sensors_temperatures"):
         All temperatures are expressed in celsius unless *fahrenheit*
         is set to True.
         """
-        def to_fahrenheit(n):
-            return (float(n) * 9 / 5) + 32
+        def convert(n):
+            if n is not None:
+                return (float(n) * 9 / 5) + 32 if fahrenheit else n
 
         ret = collections.defaultdict(list)
         rawdict = _psplatform.sensors_temperatures()
@@ -2214,12 +2270,9 @@ if hasattr(_psplatform, "sensors_temperatures"):
         for name, values in rawdict.items():
             while values:
                 label, current, high, critical = values.pop(0)
-                if fahrenheit:
-                    current = to_fahrenheit(current)
-                    if high is not None:
-                        high = to_fahrenheit(high)
-                    if critical is not None:
-                        critical = to_fahrenheit(critical)
+                current = convert(current)
+                high = convert(high)
+                critical = convert(critical)
 
                 if high and not critical:
                     critical = high
@@ -2234,6 +2287,19 @@ if hasattr(_psplatform, "sensors_temperatures"):
     __all__.append("sensors_temperatures")
 
 
+# Linux
+if hasattr(_psplatform, "sensors_fans"):
+
+    def sensors_fans():
+        """Return fans speed. Each entry is a namedtuple
+        representing a certain hardware sensor.
+        All speed are expressed in RPM (rounds per minute).
+        """
+        return _psplatform.sensors_fans()
+
+    __all__.append("sensors_fans")
+
+
 # Linux, Windows, FreeBSD
 if hasattr(_psplatform, "sensors_battery"):
 
@@ -2241,11 +2307,11 @@ if hasattr(_psplatform, "sensors_battery"):
         """Return battery information. If no battery is installed
         returns None.
 
-        - percent: battery power left as a percentage.
-        - secsleft: a rough approximation of how many seconds are left
-                    before the battery runs out of power.
-                    May be POWER_TIME_UNLIMITED or POWER_TIME_UNLIMITED.
-        - power_plugged: True if the AC power cable is connected.
+         - percent: battery power left as a percentage.
+         - secsleft: a rough approximation of how many seconds are left
+                     before the battery runs out of power. May be
+                     POWER_TIME_UNLIMITED or POWER_TIME_UNLIMITED.
+         - power_plugged: True if the AC power cable is connected.
         """
         return _psplatform.sensors_battery()
 
@@ -2291,7 +2357,7 @@ if WINDOWS:
         return _psplatform.win_service_iter()
 
     def win_service_get(name):
-        """Get a Windows service by name.
+        """Get a Windows service by *name*.
         Raise NoSuchProcess if no service with such name exists.
         """
         return _psplatform.win_service_get(name)
@@ -2307,53 +2373,47 @@ def test():  # pragma: no cover
     import datetime
 
     today_day = datetime.date.today()
-    templ = "%-10s %5s %4s %4s %7s %7s %-13s %5s %7s  %s"
-    attrs = ['pid', 'cpu_percent', 'memory_percent', 'name', 'cpu_times',
-             'create_time', 'memory_info']
+    templ = "%-10s %5s %4s %7s %7s %-13s %5s %7s  %s"
+    attrs = ['pid', 'memory_percent', 'name', 'cpu_times', 'create_time',
+             'memory_info']
     if POSIX:
         attrs.append('uids')
         attrs.append('terminal')
-    print(templ % ("USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY",
-                   "START", "TIME", "COMMAND"))
-    for p in process_iter():
-        try:
-            pinfo = p.as_dict(attrs, ad_value='')
-        except NoSuchProcess:
-            pass
-        else:
-            if pinfo['create_time']:
-                ctime = datetime.datetime.fromtimestamp(pinfo['create_time'])
-                if ctime.date() == today_day:
-                    ctime = ctime.strftime("%H:%M")
-                else:
-                    ctime = ctime.strftime("%b%d")
+    print(templ % ("USER", "PID", "%MEM", "VSZ", "RSS", "TTY", "START", "TIME",
+                   "COMMAND"))
+    for p in process_iter(attrs=attrs, ad_value=''):
+        if p.info['create_time']:
+            ctime = datetime.datetime.fromtimestamp(p.info['create_time'])
+            if ctime.date() == today_day:
+                ctime = ctime.strftime("%H:%M")
             else:
-                ctime = ''
-            cputime = time.strftime("%M:%S",
-                                    time.localtime(sum(pinfo['cpu_times'])))
-            try:
-                user = p.username()
-            except Error:
-                user = ''
-            if WINDOWS and '\\' in user:
-                user = user.split('\\')[1]
-            vms = pinfo['memory_info'] and \
-                int(pinfo['memory_info'].vms / 1024) or '?'
-            rss = pinfo['memory_info'] and \
-                int(pinfo['memory_info'].rss / 1024) or '?'
-            memp = pinfo['memory_percent'] and \
-                round(pinfo['memory_percent'], 1) or '?'
-            print(templ % (
-                user[:10],
-                pinfo['pid'],
-                pinfo['cpu_percent'],
-                memp,
-                vms,
-                rss,
-                pinfo.get('terminal', '') or '?',
-                ctime,
-                cputime,
-                pinfo['name'].strip() or '?'))
+                ctime = ctime.strftime("%b%d")
+        else:
+            ctime = ''
+        cputime = time.strftime("%M:%S",
+                                time.localtime(sum(p.info['cpu_times'])))
+        try:
+            user = p.username()
+        except Error:
+            user = ''
+        if WINDOWS and '\\' in user:
+            user = user.split('\\')[1]
+        vms = p.info['memory_info'] and \
+            int(p.info['memory_info'].vms / 1024) or '?'
+        rss = p.info['memory_info'] and \
+            int(p.info['memory_info'].rss / 1024) or '?'
+        memp = p.info['memory_percent'] and \
+            round(p.info['memory_percent'], 1) or '?'
+        print(templ % (
+            user[:10],
+            p.info['pid'],
+            memp,
+            vms,
+            rss,
+            p.info.get('terminal', '') or '?',
+            ctime,
+            cputime,
+            p.info['name'].strip() or '?'))
 
 
 del memoize, memoize_when_activated, division, deprecated_method
